clear all
close all
w0 = 2*pi*60;

%==========================================================================
%           Define o estado dos disjuntores de entrada das SEs
%==========================================================================
%      config = 1     =>  A=1/T=0/B=1
%      config = 2     =>  A=1/T=1/B=0
%      config = 3     =>  A=0/T=1/B=1
%----------------------------Configurao Escolhida------------------------
config_5142 = 1;
config_5151 = 1;
config_2100 = 1;
%--------------------------------------------------------------------------
%==========================================================================
%           Define Carga da Planta
%==========================================================================
% Fator de potencia considerado para a carga
cosfi = 0.95;  % cos(fi) das cargas

% Carga (MVA) total da Planta
Stotal = 384; % Carga total (MVA) da planta

% Carga (MVA) total da SE 5151
S5151 =   40; % Carga total da SE 5151 (SE de distribuio)

% Carga (MVA) da SE 2100
S_PN21001 = 3.5;  % demanda total DO CDC 21001 (painel 4.16 kV)
S_PN21002 = 1.5;   % demanda total do CDC 21002 (painel 0.48 V)
S2100 =  S_PN21001 + S_PN21002;  % Carga total da SE 2100

% Rejeio de carga para Operao Ilhada
%--------------------------------------------------------------------------
% Ilhamento = 0 => operando com o SIN
% Ilhamento = 1 => operando desconectado do SIN
%--------------------------------------------------------------------------
Ilhamento = 1;

if Ilhamento == 1
    krejeicao = 280/(Stotal*cosfi);
    Stotal = Stotal*krejeicao;
    S5151 = S5151*krejeicao;
    S_PN21001 = S_PN21001*krejeicao;
    S_PN21002 = S_PN21002*krejeicao;
    S2100 = S2100*krejeicao;
end
    
%==========================================================================
switch config_5142
    case 1
        A5142 = 1;
        T5142 = 0;
        B5142 = 1;
    case 2
        A5142 = 1;
        T5142 = 1;
        B5142 = 0;
    case 3
        A5142 = 0;
        T5142 = 1;
        B5142 = 1;
end

if Ilhamento == 1
    A5142 = 0;
    B5142 = 0;
    T5142 = 0;
end

switch config_5151
    case 1
        A5151 = 1;
        T5151 = 0;
        B5151 = 1;
    case 2
        A5151 = 1;
        T5151 = 1;
        B5151 = 0;
    case 3
        A5151 = 0;
        T5151 = 1;
        B5151 = 1;
end
switch config_2100
    case 1
        A2100 = 1;
        T2100 = 0;
        B2100 = 1;
    case 2
        A2100 = 1;
        T2100 = 1;
        B2100 = 0;
    case 3
        A2100 = 0;
        T2100 = 1;
        B2100 = 1;
end
%==========================================================================
%              PARAMETROS ELTRICOS DA REDE ELTRICA
%==========================================================================

% Parametros do SIN: Linhas de 345 kV
%-------LINHAS AREAS 345 KV-----------------------------------------------
aerea_R1 = [...
    0.12714E-03   0.99133E-04   0.95323E-04   0.95194E-04   0.98898E-04   0.10392E-03
    0.99133E-04   0.11707E-03   0.90971E-04   0.90884E-04   0.94225E-04   0.98780E-04
    0.95323E-04   0.90971E-04   0.11053E-03   0.87768E-04   0.90829E-04   0.95006E-04
    0.95194E-04   0.90884E-04   0.87768E-04   0.11043E-03   0.90839E-04   0.95092E-04
    0.98898E-04   0.94225E-04   0.90829E-04   0.90839E-04   0.11693E-03   0.98936E-04
    0.10392E-03   0.98780E-04   0.95006E-04   0.95092E-04   0.98936E-04   0.12696E-03]*1e3;
aerea_L1 = [...
    0.59016E-03   0.29668E-03   0.25243E-03   0.23315E-03   0.24771E-03   0.24715E-03
    0.29668E-03   0.60974E-03   0.31367E-03   0.26610E-03   0.27050E-03   0.24442E-03
    0.25243E-03   0.31367E-03   0.62438E-03   0.28647E-03   0.26499E-03   0.22886E-03
    0.23315E-03   0.26610E-03   0.28647E-03   0.62145E-03   0.30944E-03   0.24476E-03
    0.24771E-03   0.27050E-03   0.26499E-03   0.30944E-03   0.60407E-03   0.28725E-03
    0.24715E-03   0.24442E-03   0.22886E-03   0.24476E-03   0.28725E-03   0.57640E-03]*1e3/w0;
aerea_C1 = [...
    0.43372E-08  -0.97892E-09  -0.28886E-09 -0.14774E-09 -0.34136E-09 -0.58449E-09
    -0.97892E-09  0.45605E-08  -0.91776E-09 -0.29839E-09 -0.43814E-09 -0.34131E-09
    -0.28886E-09 -0.91776E-09   0.45111E-08 -0.46003E-09 -0.29838E-09 -0.14771E-09
    -0.14774E-09 -0.29839E-09  -0.46003E-09  0.45111E-08 -0.91770E-09 -0.28869E-09
    -0.34136E-09 -0.43814E-09  -0.29838E-09 -0.91770E-09  0.45607E-08 -0.97852E-09
    -0.58449E-09 -0.34131E-09  -0.14771E-09 -0.28869E-09 -0.97852E-09  0.43385E-08]*1e3/w0;

%Comprimentos das Linhas de 345 kV
L10 = 65;
L20 = 120;
L30 = 10;
%----------Equivalentes das barras 345 kV/SIN-----------------------------
% Barra 10
Z1_10 = [0.0003 0.0102/w0]*345^2/100;
Z0_10 = [0.00006 0.0031/w0]*345^2/100;

% Barra 20
Z1_20 = [0.001 0.014/w0]*345^2/100;
Z0_20 = [0.0013 0.0077/w0]*345^2/100;

%---------CABOS 345 KV CONEXO SE ENTRADA/SE PRINCIPAL--------------------
% Parametros da LT 345 kV - cabos isolados
R1_345 = [0.011508792 0.013548179];    % [R1 e R0] Ohms/km
L1_345 = [0.132013742 0.044249579]/w0; % [L1 e L0] H/km
C1_345 = [1.12E-004 1.12E-004]/w0;     % [C1 e C0] F/km

%-------SE PRINCIPAL 5142 -------------------------------------------------
% Carga na SE Principal
fi = acos(cosfi); %angulgo fi

P_138 = (Stotal - S5151)*1e6*cos(fi)/2; % Watts em cada metade da barra 138 Kv (sem SE 5151)
Q_138 = (Stotal - S5151)*1e6*sin(fi)/2; % VArs em cada metade da barra 138 kV (sem SE 5151)

%-----------DADOS TURBOS GERADORES A VAPOR---------------------------------
% Turbo Geradores a Vapor
% Vn = 25 kV
% Sn = 175 MVA
% Pn = 140 MW
% xd = 1,48; x'd = 0,196; x''d = 0,136; x0 = 0,066; x2 = 0,129
Vn_G = 25;
Sn_G = 175;
Zb_G = Vn_G*Vn_G/Sn_G;
Z1_G = [0.308 0.136*Zb_G/w0];
Z0_G = [0.308 0.066*Zb_G/w0];
Rng = 1500; % resistor de aterramento do gerador

%-------FEEDERS 138 KV - CONEXO SE PRINCIPAL/CENTRAL DE UTILIDADES--------
% 3 cabos #300mm2/fase
R138B = [0.018458226 0.093798573];    %  [R1 e R0] Ohms/km
L138B = [0.024318805 0.034055928]/w0; % [L1 e L0] H/km
C138B = [1.43E-004 1.43E-004]/w0;     % [C1 e C0] F/km


%-------FEEDERS 138 KV - CONEXO SE PRINCIPAL/SE DISTRIBUIO 5151---------
% 1 cabo #300mm2/fase
R138A = [0.05647824 0.068335118];    %  [R1 e R0] Ohms/km
L138A = [0.280836925 0.098861762]/w0; % [L1 e L0] H/km
C138A = [1.43E-004 1.43E-004]/w0;     % [C1 e C0] F/km

%-------TRAFOS 138/13.8 kV - SE DISTRIBUIO-------------------------------
Rn = 20; %resistor de aterramento (ohms)
%Estado dos disjuntores
DJ1_control = 0;

%-------SE DISTRIBUIO 5151-----------------------------------------------
P_5151 = (S5151 - S2100)*1e6*cos(fi)/2; % Watts em cada metade da barra 138 Kv (sem SE 2100)
Q_5151 = (S5151 - S2100)*1e6*sin(fi)/2; % VArs em cada metade da barra 138 kV (sem SE 2100)
Rn = 50;   %impedancia de aterramento dos trafos 138/34.5 kv

%---------FEEDERS DE 13.8 kV--CONEXO SE5151/SE2100------------------------

% ATENO VERIFICAR OS PARMETROS DOS CABOS DE 13.8

% 1 cabo #70 mm2/fase
L = 0.2; %comprimento total da linha SE5151/SE2100
L1 = 0.1; %comprimento do primeiro trecho da linha
L2 = L - L1; %comprimento de segundo trecho da linha
R13_8 = [0.234131791 0.32007399];    %  [R1 e R0] Ohms/km
L13_8 = [0.212557276 0.082579639]/w0; % [L1 e L0] H/km
C13_8 = [7.00E-005 7.00E-005]/w0;     % [C1 e C0] F/km

%-------SE AUXILIAR 2100---------------------------------------------------
% demanda total na SE 2100
P_2100 = (S_PN21001 + S_PN21001)*1e6*cos(fi)/2;
Q_2100 = (S_PN21001 + S_PN21001)*1e6*sin(fi)/2;


sim('sistema_reg2_2012_02')

% Expressa fasor em valor eficaz e tenses em kV
kk = 1/sqrt(2);
kv = 1e-3;

% Tenses e correntes na barra 10 (345 kV) do SIN
V10A = V10A*kk*kv;
V10B = V10B*kk*kv;

I10A = I10A*kk;
I10B = I10B*kk;
% Tenses e correntes na barra 20 (345 kV) do SIN

V20A = V20A*kk*kv;
V20B = V20B*kk*kv;

I20A = I20A*kk;
I20B = I20B*kk;

% Tenses e correntes na barra 30 (345 kV) do SIN
V30A = V30A*kk*kv;
V30B = V30B*kk*kv;

I30A = I30A*kk;
I30B = I30B*kk;

% Tenses e correntes na SE 5142
B5142A_V = B5142A_V*kk*kv;
B5142B_V = B5142B_V*kk*kv;
B5142C_V = B5142C_V*kk*kv;
B5142D_V = B5142D_V*kk*kv;

B5142A_I = B5142A_I*kk;
B5142B_I = B5142B_I*kk;
B5142C_I = B5142C_I*kk;
B5142D_I = B5142D_I*kk;

% Tenses e correntes na SE 5151
B5151A_V = B5151A_V*kk*kv;
B5151B_V = B5151B_V*kk*kv;
B5151C_V = B5151C_V*kk*kv;
B5151D_V = B5151D_V*kk*kv;

B5151A_I = B5151A_I*kk;
B5151B_I = B5151B_I*kk;
B5151C_I = B5151C_I*kk;
B5151D_I = B5151D_I*kk;

% Tenses e correntes na SE 2100
B2100A_V = B2100A_V*kk*kv;
B2100B_V = B2100B_V*kk*kv;
B2100C_V = B2100C_V*kk*kv;
B2100D_V = B2100D_V*kk*kv;

B2100A_I = B2100A_I*kk;
B2100B_I = B2100B_I*kk;
B2100C_I = B2100C_I*kk;
B2100D_I = B2100D_I*kk;

% Tenses e correntes nos Geradores
VGA = VGA*kk*kv;
VGB = VGB*kk*kv;
IGA = IGA*kk;
IGB = IGB*kk;

ka = 180/pi;
% tenses e correntes no 345 kV--------------------------------------------
va10A = V10A(1,1); vb10A = V10A(1,2); vc10A = V10A(1,3);
ia10A = I10A(1,1); ib10A = I10A(1,2); ic10A = I10A(1,3); in10A = ia10A+ib10A+ic10A;
va10B = V10B(1,1); vb10B = V10B(1,2); vc10B = V10B(1,3); 
ia10B = I10B(1,1); ib10B = I10B(1,2); ic10B = I10B(1,3); in10B = ia10B+ib10B+ic10B;

va20A = V20A(1,1); vb20A = V20A(1,2); vc20A = V20A(1,3);
ia20A = I20A(1,1); ib20A = I20A(1,2); ic20A = I20A(1,3); in20A = ia20A+ib20A+ic20A;
va20B = V20B(1,1); vb20B = V20B(1,2); vc20B = V20B(1,3);
ia20B = I20B(1,1); ib20B = I20B(1,2); ic20B = I20B(1,3); in20B = ia20B+ib20B+ic20B;

va30A = V30A(1,1); vb30A = V30A(1,2); vc30A = V30A(1,3);
ia30A = I30A(1,1); ib30A = I30A(1,2); ic30A = I30A(1,3); in30A = ia30A+ib30A+ic30A;
va30B = V30B(1,1); vb30B = V30B(1,2); vc30B = V30B(1,3);
ia30B = I30B(1,1); ib30B = I30B(1,2); ic30B = I30B(1,3); in30B = ia30B+ib30B+ic30B;

saidaI_345 = [...
    abs(ia10A) angle(ia10A)*ka abs(ib10A) angle(ib10A)*ka abs(ic10A) angle(ic10A)*ka abs(in10A) angle(in10A)*ka;
    abs(ia10B) angle(ia10B)*ka abs(ib10B) angle(ib10B)*ka abs(ic10B) angle(ic10B)*ka abs(in10B) angle(in10B)*ka;
    abs(ia20A) angle(ia20A)*ka abs(ib20A) angle(ib20A)*ka abs(ic20A) angle(ic20A)*ka abs(in20A) angle(in20A)*ka;
    abs(ia20B) angle(ia20B)*ka abs(ib20B) angle(ib20B)*ka abs(ic20B) angle(ic20B)*ka abs(in20B) angle(in20B)*ka;
    abs(ia30A) angle(ia30A)*ka abs(ib30A) angle(ib30A)*ka abs(ic30A) angle(ic30A)*ka abs(in30A) angle(in30A)*ka;
    abs(ia30B) angle(ia30B)*ka abs(ib30B) angle(ib30B)*ka abs(ic30B) angle(ic30B)*ka abs(in30B) angle(in30B)*ka];
saidaV_345 = [...
    abs(va10A) angle(va10A)*ka abs(vb10A) angle(vb10A)*ka abs(vc10A) angle(vc10A)*ka;
    abs(va10B) angle(va10B)*ka abs(vb10B) angle(vb10B)*ka abs(vc10B) angle(vc10B)*ka;
    abs(va20A) angle(va20A)*ka abs(vb20A) angle(vb20A)*ka abs(vc20A) angle(vc20A)*ka;
    abs(va20B) angle(va20B)*ka abs(vb20B) angle(vb20B)*ka abs(vc20B) angle(vc20B)*ka;
    abs(va30A) angle(va30A)*ka abs(vb30A) angle(vb30A)*ka abs(vc30A) angle(vc30A)*ka;
    abs(va30B) angle(va30B)*ka abs(vb30B) angle(vb30B)*ka abs(vc30B) angle(vc30B)*ka];

xlswrite('Resultados.xlsx',saidaI_345,'B4:I9')
xlswrite('Resultados.xlsx',saidaV_345,'J4:O9');

%Tenses e Correntes no 138 kV (SE 5142)-----------------------------------
va40A = B5142A_V(1,1); vb40A = B5142A_V(1,2); vc40A = B5142A_V(1,3);
ia40A = B5142A_I(1,1); ib40A = B5142A_I(1,2); ic40A = B5142A_I(1,3); in40A=ia40A+ib40A+ic40A;
va40B = B5142B_V(1,1); vb40B = B5142B_V(1,2); vc40B = B5142B_V(1,3);
ia40B = B5142B_I(1,1); ib40B = B5142B_I(1,2); ic40B = B5142B_I(1,3); in40B=ia40B+ib40B+ic40B;

va50A = B5142C_V(1,1); vb50A = B5142C_V(1,2); vc50A = B5142C_V(1,3);
ia50A = B5142C_I(1,1); ib50A = B5142C_I(1,2); ic50A = B5142C_I(1,3); in50A=ia50A+ib50A+ic50A;
va50B = B5142D_V(1,1); vb50B = B5142D_V(1,2); vc50B = B5142D_V(1,3);
ia50B = B5142D_I(1,1); ib50B = B5142D_I(1,2); ic50B = B5142D_I(1,3); in50B=ia50B+ib50B+ic50B;

saidaI_5142 = [...
    abs(ia40A) angle(ia40A)*ka abs(ib40A) angle(ib40A)*ka abs(ic40A) angle(ic40A)*ka abs(in40A) angle(in40A)*ka;
    abs(ia40B) angle(ia40B)*ka abs(ib40B) angle(ib40B)*ka abs(ic40B) angle(ic40B)*ka abs(in40B) angle(in40B)*ka;
    abs(ia50A) angle(ia50A)*ka abs(ib50A) angle(ib50A)*ka abs(ic50A) angle(ic50A)*ka abs(in50A) angle(in50A)*ka;
    abs(ia50B) angle(ia50B)*ka abs(ib50B) angle(ib50B)*ka abs(ic50B) angle(ic50B)*ka abs(in50B) angle(in50B)*ka];
saidaV_5142 = [...
    abs(va40A) angle(va40A)*ka abs(vb40A) angle(vb40A)*ka abs(vc40A) angle(vc40A)*ka;
    abs(va40B) angle(va40B)*ka abs(vb40B) angle(vb40B)*ka abs(vc40B) angle(vc40B)*ka;
    abs(va50A) angle(va50A)*ka abs(vb50A) angle(vb50A)*ka abs(vc50A) angle(vc50A)*ka;
    abs(va50B) angle(va50B)*ka abs(vb50B) angle(vb50B)*ka abs(vc50B) angle(vc50B)*ka];

xlswrite('Resultados.xlsx',saidaI_5142,'B21:I24')
xlswrite('Resultados.xlsx',saidaV_5142,'J21:O24');

%Tenses e correntes no 34,5 kV da SE Distribuio (5151)------------------
va60A = B5151A_V(1,1); vb60A = B5151A_V(1,2); vc60A = B5151A_V(1,3);
ia60A = B5151A_I(1,1); ib60A = B5151A_I(1,2); ic60A = B5151A_I(1,3); in60A=ia60A+ib60A+ic60A;
va60B = B5151B_V(1,1); vb60B = B5151B_V(1,2); vc60B = B5151B_V(1,3);
ia60B = B5151B_I(1,1); ib60B = B5151B_I(1,2); ic60B = B5151B_I(1,3); in60B=ia60B+ib60B+ic60B;

va70A = B5151C_V(1,1); vb70A = B5151C_V(1,2); vc70A = B5151C_V(1,3);
ia70A = B5151C_I(1,1); ib70A = B5151C_I(1,2); ic70A = B5151C_I(1,3); in70A=ia70A+ib70A+ic70A;
va70B = B2100D_V(1,1); vb70B = B2100D_V(1,2); vc70B = B5151D_V(1,3);
ia70B = B5151D_I(1,1); ib70B = B5151D_I(1,2); ic70B = B5151D_I(1,3); in70B=ia70B+ib70B+ic70B;

saidaI_5151 = [...
    abs(ia60A) angle(ia60A)*ka abs(ib60A) angle(ib60A)*ka abs(ic60A) angle(ic60A)*ka abs(in60A) angle(in60A)*ka;
    abs(ia60B) angle(ia60B)*ka abs(ib60B) angle(ib60B)*ka abs(ic60B) angle(ic60B)*ka abs(in60B) angle(in60B)*ka;
    abs(ia70A) angle(ia70A)*ka abs(ib70A) angle(ib70A)*ka abs(ic70A) angle(ic70A)*ka abs(in70A) angle(in70A)*ka;
    abs(ia70B) angle(ia70B)*ka abs(ib70B) angle(ib70B)*ka abs(ic70B) angle(ic70B)*ka abs(in70B) angle(in70B)*ka];
saidaV_5151 = [...
    abs(va60A) angle(va60A)*ka abs(vb60A) angle(vb60A)*ka abs(vc60A) angle(vc60A)*ka;
    abs(va60B) angle(va60B)*ka abs(vb60B) angle(vb60B)*ka abs(vc60B) angle(vc60B)*ka;
    abs(va70A) angle(va70A)*ka abs(vb70A) angle(vb70A)*ka abs(vc70A) angle(vc70A)*ka;
    abs(va70B) angle(va70B)*ka abs(vb70B) angle(vb70B)*ka abs(vc70B) angle(vc70B)*ka];

xlswrite('Resultados.xlsx',saidaI_5151,'B30:I33')
xlswrite('Resultados.xlsx',saidaV_5151,'J30:O33');


% Tenses e correntes no 34,5 da entrada SE Auxiliar (2100)----------------
va80A = B2100A_V(1,1); vb80A = B2100A_V(1,2); vc80A = B2100A_V(1,3);
ia80A = B2100A_I(1,1); ib80A = B2100A_I(1,2); ic80A = B2100A_I(1,3); in80A=ia80A+ib80A+ic80A;
va80B = B2100B_V(1,1); vb80B = B2100B_V(1,2); vc80B = B2100B_V(1,3);
ia80B = B2100B_I(1,1); ib80B = B2100B_I(1,2); ic80B = B2100B_I(1,3); in80B=ia80B+ib80B+ic80B;

va90A = B2100C_V(1,1); vb90A = B2100C_V(1,2); vc90A = B2100C_V(1,3);
ia90A = B2100C_I(1,1); ib90A = B2100C_I(1,2); ic90A = B2100C_I(1,3); in90A=ia90A+ib90A+ic90A;
va90B = B2100D_V(1,1); vb90B = B2100D_V(1,2); vc90B = B2100D_V(1,3);
ia90B = B2100D_I(1,1); ib90B = B2100D_I(1,2); ic90B = B2100D_I(1,3); in90B=ia90B+ib90B+ic90B;

saidaI_2100 = [...
    abs(ia80A) angle(ia80A)*ka abs(ib80A) angle(ib80A)*ka abs(ic80A) angle(ic80A)*ka abs(in80A) angle(in80A)*ka;
    abs(ia80B) angle(ia80B)*ka abs(ib80B) angle(ib80B)*ka abs(ic80B) angle(ic80B)*ka abs(in80B) angle(in80B)*ka;
    abs(ia90A) angle(ia90A)*ka abs(ib90A) angle(ib90A)*ka abs(ic90A) angle(ic90A)*ka abs(in90A) angle(in90A)*ka;
    abs(ia90B) angle(ia90B)*ka abs(ib90B) angle(ib90B)*ka abs(ic90B) angle(ic90B)*ka abs(in90B) angle(in90B)*ka];
saidaV_2100 = [...70
    abs(va80A) angle(va80A)*ka abs(vb80A) angle(vb80A)*ka abs(vc80A) angle(vc80A)*ka;
    abs(va80B) angle(va80B)*ka abs(vb80B) angle(vb80B)*ka abs(vc80B) angle(vc80B)*ka;
    abs(va90A) angle(va90A)*ka abs(vb90A) angle(vb90A)*ka abs(vc90A) angle(vc90A)*ka;
    abs(va90B) angle(va90B)*ka abs(vb90B) angle(vb90B)*ka abs(vc90B) angle(vc90B)*ka];

xlswrite('Resultados.xlsx',saidaI_2100,'B39:I42')
xlswrite('Resultados.xlsx',saidaV_2100,'J39:O42');

% Tenses e Correntes nos Geradores----------------------------------------
% vaGA = VGA(1,1); vbGA = VGA(1,2); vcGA = VGA(1,3);
% vaGB = VGB(1,1); vbGB = VGB(1,2); vcGB = VGB(1,3);
% iaGA = IGA(1,1); ibGA = IGA(1,2); icGA = IGA(1,3);
% iaGB = IGB(1,1); ibGB = IGB(1,2); icGB = IGB(1,3);
% 
% saidaI_G = [...
%     abs(iaGA) angle(iaGA)*ka abs(ibGA) angle(ibGA)*ka abs(icGA) angle(icGA)*ka; 
%     abs(iaGB) angle(iaGB)*ka abs(ibGB) angle(ibGB)*ka abs(icGB) angle(icGB)*ka]; 
% 
% saidaV_G = [...
%     abs(vaGA) angle(vaGA)*ka abs(vbGA) angle(vbGA)*ka abs(vcGA) angle(vcGA)*ka; 
%     abs(vaGB) angle(vaGB)*ka abs(vbGB) angle(vbGB)*ka abs(vcGB) angle(vcGB)*ka]; 
% 
% xlswrite('Resultados.xlsx',saidaI_G,'B15:I16')
% xlswrite('Resultados.xlsx',saidaV_G,'J15:O16');

